@@ -1,16 +1,18 @@
 import logging
 import traceback
-from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union
+from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union, cast
 
 from nornir.core.exceptions import NornirExecutionError
 from nornir.core.exceptions import NornirSubTaskError
 
 if TYPE_CHECKING:
-    from nornir.core.inventory import Host
     from nornir.core import Nornir
+    from nornir.core.inventory import Host
+    from nornir.core.processor import Processors
 
 
 logger = logging.getLogger(__name__)
+DEFAULT_SEVERITY_LEVEL = logging.INFO
 
 
 class Task(object):
@@ -40,27 +42,40 @@ class Task(object):
     def __init__(
         self,
         task: Callable[..., Any],
+        nornir: "Nornir",
+        global_dry_run: bool,
+        processors: "Processors",
         name: str = None,
-        severity_level: int = logging.INFO,
+        severity_level: int = DEFAULT_SEVERITY_LEVEL,
         parent_task: Optional["Task"] = None,
         **kwargs: str
     ):
-        self.name = name or task.__name__
         self.task = task
+        self.nornir = nornir
+        self.name = name or task.__name__
+        self.global_dry_run = global_dry_run
         self.parent_task = parent_task
         self.params = kwargs
         self.results = MultiResult(self.name)
         self.severity_level = severity_level
+        self.processors = processors
 
     def copy(self) -> "Task":
         return Task(
-            self.task, self.name, self.severity_level, self.parent_task, **self.params
+            self.task,
+            self.nornir,
+            self.global_dry_run,
+            self.processors,
+            self.name,
+            self.severity_level,
+            self.parent_task,
+            **self.params
         )
 
     def __repr__(self) -> str:
         return self.name
 
-    def start(self, host: "Host", nornir: "Nornir") -> "MultiResult":
+    def start(self, host: "Host") -> "MultiResult":
         """
         Run the task for the given host.
 
@@ -74,12 +89,11 @@ class Task(object):
             host (:obj:`nornir.core.task.MultiResult`): Results of the task and its subtasks
         """
         self.host = host
-        self.nornir = nornir
 
         if self.parent_task is not None:
-            self.nornir.processors.subtask_instance_started(self, host)
+            self.processors.subtask_instance_started(self, host)
         else:
-            self.nornir.processors.task_instance_started(self, host)
+            self.processors.task_instance_started(self, host)
         try:
             logger.debug("Host %r: running task %r", self.host.name, self.name)
             r = self.task(self, **self.params)
@@ -107,14 +121,19 @@ class Task(object):
             r = Result(host, exception=e, result=tb, failed=True)
 
         r.name = self.name
-        r.severity_level = logging.ERROR if r.failed else self.severity_level
+
+        if r.severity_level == DEFAULT_SEVERITY_LEVEL:
+            if r.failed:
+                r.severity_level = logging.ERROR
+            else:
+                r.severity_level = self.severity_level
 
         self.results.insert(0, r)
 
         if self.parent_task is not None:
-            self.nornir.processors.subtask_instance_completed(self, host, self.results)
+            self.processors.subtask_instance_completed(self, host, self.results)
         else:
-            self.nornir.processors.task_instance_completed(self, host, self.results)
+            self.processors.task_instance_completed(self, host, self.results)
         return self.results
 
     def run(self, task: Callable[..., Any], **kwargs: Any) -> "MultiResult":
@@ -129,7 +148,7 @@ class Task(object):
 
         This method will ensure the subtask is run only for the host in the current thread.
         """
-        if not self.host or not self.nornir:
+        if not self.host:
             msg = (
                 "You have to call this after setting host and nornir attributes. ",
                 "You probably called this from outside a nested task",
@@ -138,13 +157,21 @@ class Task(object):
 
         if "severity_level" not in kwargs:
             kwargs["severity_level"] = self.severity_level
-        run_task = Task(task, parent_task=self, **kwargs)
-        r = run_task.start(self.host, self.nornir)
-        self.results.append(r[0] if len(r) == 1 else r)
+
+        run_task = Task(
+            task,
+            self.nornir,
+            global_dry_run=self.global_dry_run,
+            processors=self.processors,
+            parent_task=self,
+            **kwargs
+        )
+        r = run_task.start(self.host)
+        self.results.append(r[0] if len(r) == 1 else cast("Result", r))
 
         if r.failed:
             # Without this we will keep running the grouped task
-            raise NornirSubTaskError(task=task, result=r)
+            raise NornirSubTaskError(task=run_task, result=r)
 
         return r
 
@@ -152,7 +179,7 @@ class Task(object):
         """
         Returns whether current task is a dry_run or not.
         """
-        return override if override is not None else self.nornir.data.dry_run
+        return override if override is not None else self.global_dry_run
 
 
 class Result(object):
@@ -163,7 +190,7 @@ class Result(object):
         changed (bool): ``True`` if the task is changing the system
         diff (obj): Diff between state of the system before/after running this task
         result (obj): Result of the task execution, see task's documentation for details
-        host (:obj:`nornir.core.inventory.Host`): Reference to the host that lead ot this result
+        host (:obj:`nornir.core.inventory.Host`): Reference to the host that lead to this result
         failed (bool): Whether the execution failed or not
         severity_level (logging.LEVEL): Severity level associated to the result of the excecution
         exception (Exception): uncaught exception thrown during the exection of the task (if any)
@@ -186,7 +213,7 @@ class Result(object):
         diff: str = "",
         failed: bool = False,
         exception: Optional[BaseException] = None,
-        severity_level: int = logging.INFO,
+        severity_level: int = DEFAULT_SEVERITY_LEVEL,
         **kwargs: Any
     ):
         self.result = result
@@ -215,41 +242,7 @@ class Result(object):
             return str(self.result)
 
 
-class AggregatedResult(Dict[str, Any]):
-    """
-    It basically is a dict-like object that aggregates the results for all devices.
-    You can access each individual result by doing ``my_aggr_result["hostname_of_device"]``.
-    """
-
-    def __init__(self, name: str, **kwargs: str):
-        self.name = name
-        super().__init__(**kwargs)
-
-    def __repr__(self) -> str:
-        return "{} ({}): {}".format(
-            self.__class__.__name__, self.name, super().__repr__()
-        )
-
-    @property
-    def failed(self) -> bool:
-        """If ``True`` at least a host failed."""
-        return any([h.failed for h in self.values()])
-
-    @property
-    def failed_hosts(self) -> Dict[str, "MultiResult"]:
-        """Hosts that failed during the execution of the task."""
-        return {h: r for h, r in self.items() if r.failed}
-
-    def raise_on_error(self) -> None:
-        """
-        Raises:
-            :obj:`nornir.core.exceptions.NornirExecutionError`: When at least a task failed
-        """
-        if self.failed:
-            raise NornirExecutionError(self)
-
-
-class MultiResult(List[Any]):
+class MultiResult(List[Result]):
     """
     It is basically is a list-like object that gives you access to the results of all subtasks for
     a particular device/task.
@@ -281,3 +274,37 @@ class MultiResult(List[Any]):
         """
         if self.failed:
             raise NornirExecutionError(self)
+
+
+class AggregatedResult(Dict[str, MultiResult]):
+    """
+    It basically is a dict-like object that aggregates the results for all devices.
+    You can access each individual result by doing ``my_aggr_result["hostname_of_device"]``.
+    """
+
+    def __init__(self, name: str, **kwargs: MultiResult):
+        self.name = name
+        super().__init__(**kwargs)
+
+    def __repr__(self) -> str:
+        return "{} ({}): {}".format(
+            self.__class__.__name__, self.name, super().__repr__()
+        )
+
+    @property
+    def failed(self) -> bool:
+        """If ``True`` at least a host failed."""
+        return any([h.failed for h in self.values()])
+
+    @property
+    def failed_hosts(self) -> Dict[str, "MultiResult"]:
+        """Hosts that failed during the execution of the task."""
+        return {h: r for h, r in self.items() if r.failed}
+
+    def raise_on_error(self) -> None:
+        """
+        Raises:
+            :obj:`nornir.core.exceptions.NornirExecutionError`: When at least a task failed
+        """
+        if self.failed:
+            raise NornirExecutionError(self)
